import os
if "EXTRA_DLL_SEARCH_PATHS" in os.environ and hasattr(os, "add_dll_directory"):
    for path in os.environ["EXTRA_DLL_SEARCH_PATHS"].split(os.pathsep):
        os.add_dll_directory(path)
import unittest
import xraylib


class TestComptonProfiles(unittest.TestCase):
    def test_pz_0(self):
        profile = xraylib.ComptonProfile(26, 0.0)
        self.assertAlmostEqual(profile, 7.060, delta=1E-6)

        profile = xraylib.ComptonProfile_Partial(26, xraylib.N1_SHELL, 0.0)
        self.assertAlmostEqual(profile, 1.550, delta=1E-6)

        profile1 = xraylib.ComptonProfile_Partial(26, xraylib.L2_SHELL, 0.0)
        profile2 = xraylib.ComptonProfile_Partial(26, xraylib.L3_SHELL, 0.0)
        self.assertAlmostEqual(profile1, profile2, delta=1E-6)
        self.assertAlmostEqual(profile1, 0.065, delta=1E-6)

    def test_pz_100(self):
        profile = xraylib.ComptonProfile(26, 100.0)
        self.assertAlmostEqual(profile, 1.8E-5, delta=1E-8)

        profile = xraylib.ComptonProfile_Partial(26, xraylib.N1_SHELL, 100.0)
        self.assertAlmostEqual(profile, 5.1E-9, delta=1E-12)

        profile1 = xraylib.ComptonProfile_Partial(26, xraylib.L2_SHELL, 100.0)
        profile2 = xraylib.ComptonProfile_Partial(26, xraylib.L3_SHELL, 100.0)
        self.assertAlmostEqual(profile1, profile2, delta=1E-10)
        self.assertAlmostEqual(profile1, 1.1E-8, delta=1E-10)

    def test_pz_50(self):
        profile = xraylib.ComptonProfile(26, 50.0)
        self.assertAlmostEqual(profile, 0.0006843950273082384, delta=1E-8)

        profile = xraylib.ComptonProfile_Partial(26, xraylib.N1_SHELL, 50.0)
        self.assertAlmostEqual(profile, 2.4322755767709126e-07, delta=1E-10)

        profile1 = xraylib.ComptonProfile_Partial(26, xraylib.L2_SHELL, 50.0)
        profile2 = xraylib.ComptonProfile_Partial(26, xraylib.L3_SHELL, 50.0)
        self.assertAlmostEqual(profile1, profile2, delta=1E-10)
        self.assertAlmostEqual(profile1, 2.026953933016568e-06, delta=1E-10)

    def test_bad_input(self):
        with self.assertRaises(ValueError):
            xraylib.ComptonProfile(0, 0.0)
        xraylib.ComptonProfile(102, 0.0)
        with self.assertRaises(ValueError):
            xraylib.ComptonProfile(103, 0.0)
        with self.assertRaises(ValueError):
            xraylib.ComptonProfile(26, -1.0)

        with self.assertRaises(ValueError):
            xraylib.ComptonProfile_Partial(0, xraylib.K_SHELL, 0.0)
        xraylib.ComptonProfile_Partial(102, xraylib.K_SHELL, 0.0)
        with self.assertRaises(ValueError):
            xraylib.ComptonProfile_Partial(103, xraylib.K_SHELL, 0.0)
        with self.assertRaises(ValueError):
            xraylib.ComptonProfile_Partial(26, xraylib.K_SHELL, -1.0)
        with self.assertRaises(ValueError):
            xraylib.ComptonProfile_Partial(26, -1, 0.0)
        with self.assertRaises(ValueError):
            xraylib.ComptonProfile_Partial(26, xraylib.N2_SHELL, 0.0)
        with self.assertRaises(TypeError):
            xraylib.ComptonProfile_Partial()
        with self.assertRaises(TypeError):
            xraylib.ComptonProfile_Partial("26", xraylib.N2_SHELL, 0.0)

if __name__ == '__main__':
    unittest.main(verbosity=2)



#!/usr/bin/env python3


import urllib.request
from io import StringIO
import lxml.etree as ET
import re

nist_url = "http://physics.nist.gov/cgi-bin/Star/compos.pl?matno="

with open("../../src/xraylib-nist-compounds-internal.h", "w") as output_int, open("../../include/xraylib-nist-compounds.h", "w") as output_header:

    header_begin = '''/*
Copyright (c) 2013-1018, Tom Schoonjans
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
    * The names of the contributors may not be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY Tom Schoonjans ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Tom Schoonjans BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

/*
 * This file was automatically generated by nist-compounds.py
 * Modify at your own risk...
 */


#ifndef XRAYLIB_NIST_COMPOUNDS_H
#define XRAYLIB_NIST_COMPOUNDS_H

#include "xraylib-error.h"

struct compoundDataNIST {
	char *name;
        int nElements;
        int *Elements;
        double *massFractions;
        double density;
};

/*
 *
 * Returns a pointer to a newly allocated struct containing
 * the requested compound on success, or NULL when the compound
 * was not found in the list. The compound is requested by providing
 * its name as argument to the function. For a list of available names,
 * use GetCompoundDataNISTList.
 *
 * The returned struct should be freed after usage with FreeCompoundDataNIST.
 *
 */
XRL_EXTERN
struct compoundDataNIST* GetCompoundDataNISTByName(const char compoundString[], xrl_error **error);

/*
 *
 * Returns a pointer to a newly allocated struct containing
 * the requested compound on success, or NULL when the compound
 * was not found in the list. The compound is requested by providing
 * its index in the internal table to the function. Typically this would
 * be done using the NIST_COMPOUND_* macros in this file.
 *
 * The returned struct should be freed after usage with FreeCompoundDataNIST.
 *
 */
XRL_EXTERN
struct compoundDataNIST* GetCompoundDataNISTByIndex(int compoundIndex, xrl_error **error);

/*
 *
 * Returns a NULL-terminated array of strings of all the compounds in the
 * internal table. If nCompounds is not NULL, it shall receive the number 
 * of compounds.
 *
 * The returned array should be freed firstly by using xrlFree to deallocate
 * all individual strings, and subsequently by using xrlFree to deallocate the array
 *
 */
XRL_EXTERN
char **GetCompoundDataNISTList(int *nCompounds, xrl_error **error);

/*
 *
 * Deallocates a pointer to a compoundDataNIST struct completely.
 * It is recommended to set the pointer to NULL after calling this function.
 *
 */
XRL_EXTERN
void FreeCompoundDataNIST(struct compoundDataNIST *compoundData);

'''

    output_header.write(header_begin)

    header_begin2 = '''/*
Copyright (c) 2013-2018, Tom Schoonjans
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
    * The names of the contributors may not be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY Tom Schoonjans ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Tom Schoonjans BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

/*
 * This file was automatically generated by nist-compounds.pl
 * Modify at your own risk...
 */

#include <xraylib-nist-compounds.h>

'''

    output_int.write(header_begin2)

    compoundData = []

    for compoundIndex, i in enumerate(range(99, 279)):
        url = '{}{:03}.html'.format(nist_url, i)
        print("URL: {}".format(url))

        try:
            response = urllib.request.urlopen(url)
            html = response.read().decode("utf-8")

            #fix html file
            html = re.sub(r"(<form action=\"\/cgi-bin\/Star\/compos.pl\" method=GET>)\n(<center>)", r"\2\n\1", html, flags=re.M)
            html = re.sub("<a\n", "\n", html)
            html = re.sub(r"<script.+<\/script>", "", html)
            parser = ET.HTMLParser()
            tree = ET.parse(StringIO(html), parser)

            searchstring = "//body/center/form/select/option[@value='{:03}']/text()".format(i)

            name = tree.xpath(searchstring)[0]
            density = tree.xpath("//body/center/table[1]/tr[1]/td[2]/text()")[0]

            nodes = tree.xpath("//body/center/table[2]/tr[@align='right']")

            Elements = []
            massFractions = []

            for node in nodes:
                Elements.append(node.xpath("td[1]/text()")[0])
                massFractions.append(node.xpath("td[2]/text()")[0])

            compoundDataSingle = dict(name=name, density=density, nElements=len(nodes), Elements=Elements, massFractions=massFractions)
            compoundData.append(compoundDataSingle)

            #macro
            macro = name.upper()
            macro = re.sub(', ', '_', macro)
            macro = re.sub(' ', '_', macro)
            macro = re.sub('-', '_', macro)
            macro = re.sub(r'\(', '', macro)
            macro = re.sub(r'\)', '', macro)
            macro = re.sub(',', '', macro)
            macro = re.sub('/', '', macro)

            output_header.write("#define NIST_COMPOUND_{} {}\n".format(macro, compoundIndex))
        except Exception as e:
            print(e)
            raise



    output_int.write("static const int nCompoundDataNISTList = {};\n".format(len(compoundData)))

    for i, cd in enumerate(compoundData):
        output_int.write("static int __CompoundDataNISTList_Elements_{}[] = {{{}}};\n".format(i, ", ".join(cd['Elements'])))
        output_int.write("static double __CompoundDataNISTList_massFractions_{}[] = {{{}}};\n".format(i, ", ".join(cd['massFractions'])))

    output_int.write("static const struct compoundDataNIST compoundDataNISTList[] = {\n")

    #output_int.write("{{\"{}\" ,{}, __CompoundDataNISTList_Elements_{}, __CompoundDataNISTList_massFractions_{}, {}}}".format(cd['name'], cd['nElements'], i, i, cd['density']))
    ls = ["{{\"{}\" ,{}, __CompoundDataNISTList_Elements_{}, __CompoundDataNISTList_massFractions_{}, {:f}}}".format(cd['name'], cd['nElements'], i, i, float(cd['density'])) for i, cd in enumerate(compoundData)]
    output_int.write(",\n".join(ls) + "\n")

    output_int.write("};\n")

    output_header.write("\n#endif\n")




#!/usr/bin/env python3

# taken from https://thispointer.com/python-how-to-get-list-of-files-in-directory-and-sub-directories/

import os
import sys
import shutil

'''
    For the given path, get the List of all files in the directory tree 
'''
def getListOfFiles(dirName):
    # create a list of file and sub directories 
    # names in the given directory 
    listOfFile = os.listdir(dirName)
    allFiles = list()
    # Iterate over all the entries
    for entry in listOfFile:
        # Create full path
        fullPath = os.path.join(dirName, entry)
        # If entry is a directory then get the list of files in this directory 
        if os.path.isdir(fullPath):
            allFiles = allFiles + getListOfFiles(fullPath)
        else:
            allFiles.append(fullPath)
                
    return allFiles

build_root = os.environ['MESON_BUILD_ROOT']
install_destdir_prefix = os.environ['MESON_INSTALL_DESTDIR_PREFIX']
files = getListOfFiles(build_root)
files = list(filter(lambda file: os.path.basename(file) == 'xraylib.mod', files))
if not files:
    sys.exit('xraylib.mod was not found!')

shutil.copy(files[0], os.path.join(install_destdir_prefix, 'include', 'xraylib'))






import os
if "EXTRA_DLL_SEARCH_PATHS" in os.environ and hasattr(os, "add_dll_directory"):
    for path in os.environ["EXTRA_DLL_SEARCH_PATHS"].split(os.pathsep):
        os.add_dll_directory(path)
import unittest
import xraylib
import math


class TestCrystalDiffraction(unittest.TestCase):
    def test_crystal_diffraction(self):
        crystals_list = xraylib.Crystal_GetCrystalsList()
        self.assertEqual(len(crystals_list), 38)
        for crystal_name in crystals_list:
            cs = xraylib.Crystal_GetCrystal(crystal_name)
            self.assertEqual(crystal_name, cs['name'])

        with self.assertRaises(ValueError):
            cs = xraylib.Crystal_GetCrystal(None)

        with self.assertRaises(ValueError):
            cs = xraylib.Crystal_GetCrystal("non-existent-crystal")

        cs = xraylib.Crystal_GetCrystal("Diamond") 

        cs_copy = xraylib.Crystal_MakeCopy(cs)

        with self.assertRaises(ValueError):
            xraylib.Crystal_AddCrystal(cs)

        with self.assertRaises(ValueError):
            xraylib.Crystal_AddCrystal(cs_copy)

        cs_copy['name'] = "Diamond-copy"
        xraylib.Crystal_AddCrystal(cs_copy)

        cs_copy['name'] = 20012016
        with self.assertRaises(TypeError):
            xraylib.Crystal_AddCrystal(cs_copy)

        cs_copy['name'] = "Diamond-copy"

        cs_copy['atom'] = list()
        with self.assertRaises(TypeError):
            xraylib.Crystal_AddCrystal(cs_copy)

        cs_copy['atom'] = (25, "jkewjfpwejffj", None, )
        with self.assertRaises(TypeError):
            xraylib.Crystal_AddCrystal(cs_copy)

        del cs_copy['atom']

        with self.assertRaises(KeyError):
            xraylib.Crystal_AddCrystal(cs_copy)

        crystals_list = xraylib.Crystal_GetCrystalsList()
        self.assertEqual(len(crystals_list), 39)

        for crystal_name in crystals_list:
            cs = xraylib.Crystal_GetCrystal(crystal_name)
            self.assertEqual(crystal_name, cs['name'])

        current_ncrystals = len(crystals_list)

        for i in range(xraylib.CRYSTALARRAY_MAX):
            cs_copy = xraylib.Crystal_MakeCopy(cs)
            cs_copy['name'] = "Diamond copy {}".format(i)
            if current_ncrystals < xraylib.CRYSTALARRAY_MAX:
                xraylib.Crystal_AddCrystal(cs_copy)
                current_ncrystals = current_ncrystals + 1
                self.assertEqual(len(xraylib.Crystal_GetCrystalsList()), current_ncrystals)
            else:
                with self.assertRaises(RuntimeError):
                    xraylib.Crystal_AddCrystal(cs_copy)
                self.assertEqual(len(xraylib.Crystal_GetCrystalsList()), xraylib.CRYSTALARRAY_MAX)

        cs = xraylib.Crystal_GetCrystal("Diamond") 

        # Bragg angle
        angle = xraylib.Bragg_angle(cs, 10.0, 1, 1, 1)
        self.assertAlmostEqual(angle, 0.3057795845795849)

        with self.assertRaises(TypeError):
            angle = xraylib.Bragg_angle(None, 10.0, 1, 1, 1)

        with self.assertRaises(ValueError):
            angle = xraylib.Bragg_angle(cs, -10.0, 1, 1, 1)

        with self.assertRaises(TypeError):
            angle = xraylib.Bragg_angle(cs, 1, 1, 1)

        
	# Q_scattering_amplitude
        tmp = xraylib.Q_scattering_amplitude(cs, 10.0, 1, 1, 1, math.pi/4.0)
        self.assertAlmostEqual(tmp, 0.19184445408324474)

        tmp = xraylib.Q_scattering_amplitude(cs, 10.0, 0, 0, 0, math.pi/4.0)
        self.assertEqual(tmp, 0.0)

        # Atomic factors
        (f0, f_prime, f_prime2) = xraylib.Atomic_Factors(26, 10.0, 1.0, 10.0)
        self.assertAlmostEqual(f0, 65.15)
        self.assertAlmostEqual(f_prime, -0.22193271025027966)
        self.assertAlmostEqual(f_prime2, 22.420270655080493)

        with self.assertRaises(ValueError):
            (f0, f_prime, f_prime2) = xraylib.Atomic_Factors(-10, 10.0, 1.0, 10.0)

        # unit cell volume
        tmp = xraylib.Crystal_UnitCellVolume(cs)
        self.assertAlmostEqual(tmp, 45.376673902751)

        # crystal dspacing
        tmp = xraylib.Crystal_dSpacing(cs, 1, 1, 1)
        self.assertAlmostEqual(tmp, 2.0592870875248344)

        del cs

if __name__ == '__main__':
    unittest.main(verbosity=2)



import numpy as np
import xraylib_np as xrl_np
import matplotlib.pyplot as plt
from matplotlib import gridspec

Z = np.arange(3, 99)

yields_bad = xrl_np.FluorYield(Z, np.array([xrl_np.K_SHELL])).squeeze()

yields_good = []

with open('fluor_yield_revised.dat', 'r') as file:
    for line in file:
        [myZ, y] = line.split()
        yields_good.append(y)
    yields_good = np.asarray(yields_good, dtype=np.float64)


fig = plt.figure(figsize=(7, 10))
gs = gridspec.GridSpec(3, 1, height_ratios=[3, 1, 1])
ax = fig.add_subplot(gs[0])
ax.set_xlabel('Atomic number')
ax.set_ylabel('Fluorescence yield')
plot_bad, = ax.plot(Z, yields_bad, color='r', marker='o', label='Fitted Hubbell')
plot_good, = ax.plot(Z, yields_good[0:Z.shape[0]], color='g', marker='o', label='Recommended Hubbell')
ax.legend(handles=[plot_bad, plot_good], loc=4)

ax = fig.add_subplot(gs[1])
ax.set_ylabel('Absolute diff')
ax.plot(Z, (yields_good[0:Z.shape[0]] - yields_bad), color='b', marker='o')

ax = fig.add_subplot(gs[2])
ax.set_ylabel('Relative diff')
ax.plot(Z, (yields_good[0:Z.shape[0]] - yields_bad) / yields_good[0:Z.shape[0]], color='b', marker='o')

plt.draw()
plt.savefig('yield-comparison.png')
plt.close()



import os
if "EXTRA_DLL_SEARCH_PATHS" in os.environ and hasattr(os, "add_dll_directory"):
    for path in os.environ["EXTRA_DLL_SEARCH_PATHS"].split(os.pathsep):
        os.add_dll_directory(path)
import unittest
import xraylib


class TestNISTCompounds(unittest.TestCase):
    def test_good(self):
        list = xraylib.GetCompoundDataNISTList()
        self.assertEqual(len(list), 180)
        for i, v in enumerate(list):
            cdn = xraylib.GetCompoundDataNISTByIndex(i)
            self.assertEqual(cdn['name'], v)
            cdn = xraylib.GetCompoundDataNISTByName(v)
            self.assertEqual(cdn['name'], v)

        cdn = xraylib.GetCompoundDataNISTByIndex(5)
        self.assertEqual(cdn['nElements'], 4)
        self.assertEqual(cdn['density'], 0.001205)
        self.assertEqual(cdn['Elements'], (6, 7, 8, 18))
        self.assertAlmostEqual(cdn['massFractions'], (0.000124, 0.755267, 0.231781, 0.012827))
        self.assertEqual(cdn['name'], 'Air, Dry (near sea level)')

        cdn = xraylib.GetCompoundDataNISTByName('Air, Dry (near sea level)')
        self.assertEqual(cdn['nElements'], 4)
        self.assertEqual(cdn['density'], 0.001205)
        self.assertEqual(cdn['Elements'], (6, 7, 8, 18))
        self.assertAlmostEqual(cdn['massFractions'], (0.000124, 0.755267, 0.231781, 0.012827))
        self.assertEqual(cdn['name'], 'Air, Dry (near sea level)')

    def test_bad(self):
        with self.assertRaises(ValueError):
            xraylib.GetCompoundDataNISTByName("jwjfpfj")
        with self.assertRaises(TypeError):
            xraylib.GetCompoundDataNISTByName(0)
        with self.assertRaises(ValueError):
            xraylib.GetCompoundDataNISTByName(None)
        with self.assertRaises(ValueError):
            xraylib.GetCompoundDataNISTByIndex(-1)
        with self.assertRaises(ValueError):
            xraylib.GetCompoundDataNISTByIndex(180)
        with self.assertRaises(TypeError):
            xraylib.GetCompoundDataNISTByIndex(None)
        with self.assertRaises(TypeError):
            xraylib.GetCompoundDataNISTByIndex("jpwjfpfwj")

if __name__ == '__main__':
    unittest.main(verbosity=2)



#!/usr/bin/env python3

import sys
if sys.version_info.minor < 6:
	raise Exception("Execute this script with at least python 3.6 to ensure all dicts are ordered!")

XRL_FUNCTIONS = {
		'AtomicWeight': {'Z': int},
		'ElementDensity': {'Z': int},
		'CS_Total': {'Z': int, 'E': float},
		'CS_Photo': {'Z': int, 'E': float},
		'CS_Rayl': {'Z': int, 'E': float},
		'CS_Compt': {'Z': int, 'E': float},
		'CS_Energy': {'Z': int, 'E': float},
		'CS_KN': {'E': float},
		'CSb_Total': {'Z': int, 'E': float},
		'CSb_Photo': {'Z': int, 'E': float},
		'CSb_Rayl': {'Z': int, 'E': float},
		'CSb_Compt': {'Z': int, 'E': float},
		'DCS_Thoms': {'theta': float},
		'DCS_KN': {'E':float, 'theta': float},
		'DCS_Rayl': {'Z': int, 'E':float, 'theta': float},
		'DCS_Compt': {'Z': int, 'E':float, 'theta': float},
		'DCSb_Rayl': {'Z': int, 'E':float, 'theta': float},
		'DCSb_Compt': {'Z': int, 'E':float, 'theta': float},
		'DCSP_Thoms': {'theta': float, 'phi': float},
		'DCSP_KN': {'E': float, 'theta': float, 'phi': float},
		'DCSP_Rayl': {'Z': int, 'E': float, 'theta': float, 'phi': float},
		'DCSP_Compt': {'Z': int, 'E': float, 'theta': float, 'phi': float},
		'DCSPb_Rayl': {'Z': int, 'E': float, 'theta': float, 'phi': float},
		'DCSPb_Compt': {'Z': int, 'E': float, 'theta': float, 'phi': float},
		'FF_Rayl': {'Z': int, 'q': float},
		'SF_Compt': {'Z': int, 'q': float},
		'MomentTransf': {'E': float, 'theta': float},
		'LineEnergy': {'Z': int, 'line': int},
		'FluorYield': {'Z': int, 'shell': int},
		'CosKronTransProb': {'Z': int, 'trans': int},
		'EdgeEnergy': {'Z': int, 'shell': int},
		'JumpFactor': {'Z': int, 'shell': int},
		'CS_FluorLine': {'Z': int, 'line': int, 'E': float},
		'CSb_FluorLine': {'Z': int, 'line': int, 'E': float},
		'CS_FluorShell': {'Z': int, 'shell': int, 'E': float},
		'CSb_FluorShell': {'Z': int, 'shell': int, 'E': float},
		'RadRate': {'Z': int, 'line': int},
		'ComptonEnergy': {'E0': float, 'theta': float},
		'Fi': {'Z': int, 'E': float},
		'Fii': {'Z': int, 'E': float},
		'CS_Photo_Total': {'Z': int, 'E': float},
		'CSb_Photo_Total': {'Z': int, 'E': float},
		'CS_Photo_Partial': {'Z': int, 'shell': int, 'E': float},
		'CSb_Photo_Partial': {'Z': int, 'shell': int, 'E': float},
		'CS_Total_Kissel': {'Z': int, 'E': float},
		'CSb_Total_Kissel': {'Z': int, 'E': float},
		'ComptonProfile': {'Z': int, 'pz': float},
		'ComptonProfile_Partial': {'Z': int, 'shell': int, 'pz': float},
		'ElectronConfig': {'Z': int, 'shell': int},
		'ElectronConfig_Biggs': {'Z': int, 'shell': int},
		'AtomicLevelWidth': {'Z': int, 'shell': int},
		'AugerRate': {'Z': int, 'auger_trans': int},
		'AugerYield': {'Z': int, 'shell': int},
		'CS_FluorLine_Kissel': {'Z': int, 'line': int, 'E': float},
		'CSb_FluorLine_Kissel': {'Z': int, 'line': int, 'E': float},
		'CS_FluorLine_Kissel_Cascade': {'Z': int, 'line': int, 'E': float},
		'CSb_FluorLine_Kissel_Cascade': {'Z': int, 'line': int, 'E': float},
		'CS_FluorLine_Kissel_Nonradiative_Cascade': {'Z': int, 'line': int, 'E': float},
		'CSb_FluorLine_Kissel_Nonradiative_Cascade': {'Z': int, 'line': int, 'E': float},
		'CS_FluorLine_Kissel_Radiative_Cascade': {'Z': int, 'line': int, 'E': float},
		'CSb_FluorLine_Kissel_Radiative_Cascade': {'Z': int, 'line': int, 'E': float},
		'CS_FluorLine_Kissel_no_Cascade': {'Z': int, 'line': int, 'E': float},
		'CSb_FluorLine_Kissel_no_Cascade': {'Z': int, 'line': int, 'E': float},
		'CS_FluorShell_Kissel': {'Z': int, 'shell': int, 'E': float},
		'CSb_FluorShell_Kissel': {'Z': int, 'shell': int, 'E': float},
		'CS_FluorShell_Kissel_Cascade': {'Z': int, 'shell': int, 'E': float},
		'CSb_FluorShell_Kissel_Cascade': {'Z': int, 'shell': int, 'E': float},
		'CS_FluorShell_Kissel_Nonradiative_Cascade': {'Z': int, 'shell': int, 'E': float},
		'CSb_FluorShell_Kissel_Nonradiative_Cascade': {'Z': int, 'shell': int, 'E': float},
		'CS_FluorShell_Kissel_Radiative_Cascade': {'Z': int, 'shell': int, 'E': float},
		'CSb_FluorShell_Kissel_Radiative_Cascade': {'Z': int, 'shell': int, 'E': float},
		'CS_FluorShell_Kissel_no_Cascade': {'Z': int, 'shell': int, 'E': float},
		'CSb_FluorShell_Kissel_no_Cascade': {'Z': int, 'shell': int, 'E': float},
		'PL1_pure_kissel': {'Z': int, 'E': float},
		'PL1_rad_cascade_kissel': {'Z': int, 'E': float, 'PK': float},
		'PL1_auger_cascade_kissel': {'Z': int, 'E': float, 'PK': float},
		'PL1_full_cascade_kissel': {'Z': int, 'E': float, 'PK': float},
		'PL2_pure_kissel': {'Z': int, 'E': float, 'PL1': float},
		'PL2_rad_cascade_kissel': {'Z': int, 'E': float, 'PK': float, 'PL1': float},
		'PL2_auger_cascade_kissel': {'Z': int, 'E': float, 'PK': float, 'PL1': float},
		'PL2_full_cascade_kissel': {'Z': int, 'E': float, 'PK': float, 'PL1': float},
		'PL3_pure_kissel': {'Z': int, 'E': float, 'PL1': float, 'PL2': float},
		'PL3_rad_cascade_kissel': {'Z': int, 'E': float, 'PK': float, 'PL1': float, 'PL2': float},
		'PL3_auger_cascade_kissel': {'Z': int, 'E': float, 'PK': float, 'PL1': float, 'PL2': float},
		'PL3_full_cascade_kissel': {'Z': int, 'E': float, 'PK': float, 'PL1': float, 'PL2': float},
		'PM1_pure_kissel': {'Z': int, 'E': float},
		'PM1_rad_cascade_kissel': {'Z': int, 'E': float, 'PK': float, 'PL1': float, 'PL2': float, 'PL3': float},
		'PM1_auger_cascade_kissel': {'Z': int, 'E': float, 'PK': float, 'PL1': float, 'PL2': float, 'PL3': float},
		'PM1_full_cascade_kissel': {'Z': int, 'E': float, 'PK': float, 'PL1': float, 'PL2': float, 'PL3': float},
		'PM2_pure_kissel': {'Z': int, 'E': float, 'PM1': float},
		'PM2_rad_cascade_kissel': {'Z': int, 'E': float, 'PK': float, 'PL1': float, 'PL2': float, 'PL3': float, 'PM1': float},
		'PM2_auger_cascade_kissel': {'Z': int, 'E': float, 'PK': float, 'PL1': float, 'PL2': float, 'PL3': float, 'PM1': float},
		'PM2_full_cascade_kissel': {'Z': int, 'E': float, 'PK': float, 'PL1': float, 'PL2': float, 'PL3': float, 'PM1': float},
		'PM3_pure_kissel': {'Z': int, 'E': float, 'PM1': float, 'PM2': float},
		'PM3_rad_cascade_kissel': {'Z': int, 'E': float, 'PK': float, 'PL1': float, 'PL2': float, 'PL3': float, 'PM1': float, 'PM2': float},
		'PM3_auger_cascade_kissel': {'Z': int, 'E': float, 'PK': float, 'PL1': float, 'PL2': float, 'PL3': float, 'PM1': float, 'PM2': float},
		'PM3_full_cascade_kissel': {'Z': int, 'E': float, 'PK': float, 'PL1': float, 'PL2': float, 'PL3': float, 'PM1': float, 'PM2': float},
		'PM4_pure_kissel': {'Z': int, 'E': float, 'PM1': float, 'PM2': float, 'PM3': float},
		'PM4_rad_cascade_kissel': {'Z': int, 'E': float, 'PK': float, 'PL1': float, 'PL2': float, 'PL3': float, 'PM1': float, 'PM2': float, 'PM3': float},
		'PM4_auger_cascade_kissel': {'Z': int, 'E': float, 'PK': float, 'PL1': float, 'PL2': float, 'PL3': float, 'PM1': float, 'PM2': float, 'PM3': float},
		'PM4_full_cascade_kissel': {'Z': int, 'E': float, 'PK': float, 'PL1': float, 'PL2': float, 'PL3': float, 'PM1': float, 'PM2': float, 'PM3': float},
		'PM5_pure_kissel': {'Z': int, 'E': float, 'PM1': float, 'PM2': float, 'PM3': float, 'PM4': float},
		'PM5_rad_cascade_kissel': {'Z': int, 'E': float, 'PK': float, 'PL1': float, 'PL2': float, 'PL3': float, 'PM1': float, 'PM2': float, 'PM3': float, 'PM4': float},
		'PM5_auger_cascade_kissel': {'Z': int, 'E': float, 'PK': float, 'PL1': float, 'PL2': float, 'PL3': float, 'PM1': float, 'PM2': float, 'PM3': float, 'PM4': float},
		'PM5_full_cascade_kissel': {'Z': int, 'E': float, 'PK': float, 'PL1': float, 'PL2': float, 'PL3': float, 'PM1': float, 'PM2': float, 'PM3': float, 'PM4': float},

		'CS_Total_CP': {'compound': str, 'E': float},
		'CS_Photo_CP': {'compound': str, 'E': float},
		'CS_Rayl_CP': {'compound': str, 'E': float},
		'CS_Compt_CP': {'compound': str, 'E': float},
		'CS_Energy_CP': {'compound': str, 'E': float},
		'CSb_Total_CP': {'compound': str, 'E': float},
		'CSb_Photo_CP': {'compound': str, 'E': float},
		'CSb_Rayl_CP': {'compound': str, 'E': float},
		'CSb_Compt_CP': {'compound': str, 'E': float},
		'DCS_Rayl_CP': {'compound': str, 'E': float, 'theta': float},
		'DCS_Compt_CP': {'compound': str, 'E': float, 'theta': float},
		'DCSb_Rayl_CP': {'compound': str, 'E': float, 'theta': float},
		'DCSb_Compt_CP': {'compound': str, 'E': float, 'theta': float},
		'DCSP_Rayl_CP': {'compound': str, 'E': float, 'theta': float, 'phi': float},
		'DCSP_Compt_CP': {'compound': str, 'E': float, 'theta': float, 'phi': float},
		'DCSPb_Rayl_CP': {'compound': str, 'E': float, 'theta': float, 'phi': float},
		'DCSPb_Compt_CP': {'compound': str, 'E': float, 'theta': float, 'phi': float},
		'CS_Photo_Total_CP': {'compound': str, 'E': float},
		'CS_Total_Kissel_CP': {'compound': str, 'E': float},
		'CSb_Photo_Total_CP': {'compound': str, 'E': float},
		'CSb_Total_Kissel_CP': {'compound': str, 'E': float},
        'Refractive_Index_Re': {'compound': str, 'E': float, 'density': float},
        'Refractive_Index_Im': {'compound': str, 'E': float, 'density': float}
	}

def generate_iface_arg_for_str(arg_name: str) -> str:
    return f"{arg_name}:string"

def generate_iface_arg_for_float(arg_name: str) -> str:
    return f"{arg_name}:double"

def generate_iface_arg_for_int(arg_name: str) -> str:
    return f"{arg_name}:longint"

GENERATE_IFACE_ARG = {
	str: generate_iface_arg_for_str,
	float: generate_iface_arg_for_float,
	int: generate_iface_arg_for_int
}

def generate_iface_c_arg_for_str(arg_name: str) -> str:
    return f"{arg_name}:PAnsiChar"

GENERATE_IFACE_C_ARG = {
	str: generate_iface_c_arg_for_str,
	float: generate_iface_arg_for_float,
	int: generate_iface_arg_for_int
}

def generate_additional_var_for_str(arg_name: str) -> str:
    return f"{arg_name}_c:PAnsiChar;\n"

GENERATE_ADDITIONAL_VAR = {
        str: generate_additional_var_for_str,        
	float: lambda arg_name: "",
	int: lambda arg_name: ""
}

def generate_preprocess_for_str(arg_name: str) -> str:
    return f"{arg_name}_c := PAnsichar(AnsiString({arg_name}));\n"

GENERATE_PREPROCESS = {
	str: generate_preprocess_for_str,
	float: lambda arg_name: "",
	int: lambda arg_name: ""
}

GENERATE_CALL = {
	str: lambda arg_name: f"{arg_name}_c",
	float: lambda arg_name: f"{arg_name}",
	int: lambda arg_name: f"{arg_name}"
}

def process_function(name: str, iface_f, impl_f) -> str:

    args = XRL_FUNCTIONS[name]
    iface = "function {}({}):double;\n".format(name, '; '.join([GENERATE_IFACE_ARG[arg_type](arg_name) for arg_name, arg_type in args.items()]))
    iface_f.write(iface)
    iface_c = "function {}_C({}; error:PPxrl_error):double;cdecl;external External_library name '{}';\n".format(name, '; '.join([GENERATE_IFACE_C_ARG[arg_type](arg_name) for arg_name, arg_type in args.items()]), name)
    impl_f.write(iface_c)
    additional_vars = ''.join([GENERATE_ADDITIONAL_VAR[arg_type](arg_name) for arg_name, arg_type in args.items()])
    preprocess = ''.join([GENERATE_PREPROCESS[arg_type](arg_name) for arg_name, arg_type in args.items()])
    arg_list =  ', '.join([GENERATE_CALL[arg_type](arg_name) for arg_name, arg_type in args.items()])
    impl = f'''
{iface}
var
    error: Pxrl_error;
    rv: double;
    {additional_vars}
begin
    error := nil;
    {preprocess}
    rv := {name}_C({arg_list}, @error);
    process_error(error);
    result := rv
end;

'''
    impl_f.write(impl)

with open('xraylib_iface.pas', 'w') as iface, open('xraylib_impl.pas', 'w') as impl:
    iface.write('{ This file has been generated automatically using generate-code.py}\n\n')
    impl.write('{ This file has been generated automatically using generate-code.py}\n\n')
    for function in XRL_FUNCTIONS.keys():
        process_function(function, iface, impl)



import os
if "EXTRA_DLL_SEARCH_PATHS" in os.environ and hasattr(os, "add_dll_directory"):
    for path in os.environ["EXTRA_DLL_SEARCH_PATHS"].split(os.pathsep):
        os.add_dll_directory(path)
import unittest
import xraylib


class TestCompoundParser(unittest.TestCase):
    def test_good_compounds(self):
        self.assertIsInstance(xraylib.CompoundParser("C19H29COOH"), dict)
        self.assertIsInstance(xraylib.CompoundParser("C12H10"), dict)
        self.assertIsInstance(xraylib.CompoundParser("C12H6O2"), dict)
        self.assertIsInstance(xraylib.CompoundParser("C6H5Br"), dict)
        self.assertIsInstance(xraylib.CompoundParser("C3H4OH(COOH)3"), dict)
        self.assertIsInstance(xraylib.CompoundParser("HOCH2CH2OH"), dict)
        self.assertIsInstance(xraylib.CompoundParser("C5H11NO2"), dict)
        self.assertIsInstance(xraylib.CompoundParser("CH3CH(CH3)CH3"), dict)
        self.assertIsInstance(xraylib.CompoundParser("NH2CH(C4H5N2)COOH"), dict)
        self.assertIsInstance(xraylib.CompoundParser("H2O"), dict)
        self.assertIsInstance(xraylib.CompoundParser("Ca5(PO4)3F"), dict)
        self.assertIsInstance(xraylib.CompoundParser("Ca5(PO4)3OH"), dict)
        self.assertIsInstance(xraylib.CompoundParser("Ca5.522(PO4.48)3OH"), dict)
        self.assertIsInstance(xraylib.CompoundParser("Ca5.522(PO.448)3OH"), dict)

    def test_bad_compounds(self):
        with self.assertRaises(ValueError):
            xraylib.CompoundParser("CuI2ww")
        with self.assertRaises(ValueError):
            xraylib.CompoundParser("0C")
        with self.assertRaises(ValueError):
            xraylib.CompoundParser("2O")
        with self.assertRaises(ValueError):
            xraylib.CompoundParser("13Li")
        with self.assertRaises(ValueError):
            xraylib.CompoundParser("2(NO3)")
        with self.assertRaises(ValueError):
            xraylib.CompoundParser("H(2)")
        with self.assertRaises(ValueError):
            xraylib.CompoundParser("Ba(12)")
        with self.assertRaises(ValueError):
            xraylib.CompoundParser("Cr(5)3")
        with self.assertRaises(ValueError):
            xraylib.CompoundParser("Pb(13)2")
        with self.assertRaises(ValueError):
            xraylib.CompoundParser("Au(22)11")
        with self.assertRaises(ValueError):
            xraylib.CompoundParser("Au11(H3PO4)2)")
        with self.assertRaises(ValueError):
            xraylib.CompoundParser("Au11(H3PO4))2")
        with self.assertRaises(ValueError):
            xraylib.CompoundParser("Au(11(H3PO4))2")
        with self.assertRaises(ValueError):
            xraylib.CompoundParser("Ca5.522(PO.44.8)3OH")
        with self.assertRaises(ValueError):
            xraylib.CompoundParser("Ba[12]")
        with self.assertRaises(ValueError):
            xraylib.CompoundParser("Auu1")
        with self.assertRaises(ValueError):
            xraylib.CompoundParser("AuL1")
        with self.assertRaises(ValueError):
            xraylib.CompoundParser(None)
        with self.assertRaises(ValueError):
            xraylib.CompoundParser("  ")
        with self.assertRaises(ValueError):
            xraylib.CompoundParser("\t")
        with self.assertRaises(ValueError):
            xraylib.CompoundParser("\n")
        with self.assertRaises(ValueError):
            xraylib.CompoundParser("Au L1")
        with self.assertRaises(ValueError):
            xraylib.CompoundParser("Au\tFe")
        with self.assertRaises(TypeError):
            xraylib.CompoundParser(26)

    def test_H2SO4(self):
        cd = xraylib.CompoundParser('H2SO4')
        self.assertEqual(cd['nElements'], 3)
        self.assertEqual(cd['molarMass'], 98.09)
        self.assertEqual(cd['nAtomsAll'], 7.0)
        self.assertEqual(cd['Elements'], (1, 8, 16))
        self.assertAlmostEqual(cd['massFractions'], (0.02059333265368539, 0.6524620246712203, 0.32694464267509427))
        self.assertAlmostEqual(cd['nAtoms'], (2.0, 4.0, 1.0))

class TestSymbolToAtomicNumber(unittest.TestCase):
    def test_Fe(self):
        self.assertEqual(xraylib.SymbolToAtomicNumber('Fe'), 26)

    def test_bad_symbol(self):
        with self.assertRaises(ValueError):
            xraylib.SymbolToAtomicNumber('Uu')

    def test_bad_type(self):
        with self.assertRaises(TypeError):
            xraylib.SymbolToAtomicNumber(26)
        with self.assertRaises(ValueError):
            xraylib.SymbolToAtomicNumber(None)

class TestAtomicNumberToSymbol(unittest.TestCase):
    def test_Fe(self):
        self.assertEqual(xraylib.AtomicNumberToSymbol(26), 'Fe')

    def test_bad_symbol(self):
        with self.assertRaises(ValueError):
            xraylib.AtomicNumberToSymbol(-2)
        with self.assertRaises(ValueError):
            xraylib.AtomicNumberToSymbol(108)

    def test_bad_type(self):
        with self.assertRaises(TypeError):
            xraylib.AtomicNumberToSymbol("26")
        with self.assertRaises(TypeError):
            xraylib.AtomicNumberToSymbol("Fe")
        with self.assertRaises(TypeError):
            xraylib.AtomicNumberToSymbol(None)

class TestCrossValidation(unittest.TestCase):
    def test(self):
        for Z in range(1, 108):
            symbol = xraylib.AtomicNumberToSymbol(Z)
            self.assertEqual(xraylib.SymbolToAtomicNumber(symbol), Z)

if __name__ == '__main__':
    unittest.main(verbosity=2)



#!/usr/bin/env python3

import sys
if sys.version_info.minor < 6:
    raise Exception("Execute this script with at least python 3.6 to ensure all dicts are ordered!")

XRL_FUNCTIONS = {
    'AtomicWeight': {'Z': int},
    'ElementDensity': {'Z': int},
    'CS_Total': {'Z': int, 'E': float},
    'CS_Photo': {'Z': int, 'E': float},
    'CS_Rayl': {'Z': int, 'E': float},
    'CS_Compt': {'Z': int, 'E': float},
    'CS_Energy': {'Z': int, 'E': float},
    'CS_KN': {'E': float},
    'CSb_Total': {'Z': int, 'E': float},
    'CSb_Photo': {'Z': int, 'E': float},
    'CSb_Rayl': {'Z': int, 'E': float},
    'CSb_Compt': {'Z': int, 'E': float},
    'DCS_Thoms': {'theta': float},
    'DCS_KN': {'E':float, 'theta': float},
    'DCS_Rayl': {'Z': int, 'E':float, 'theta': float},
    'DCS_Compt': {'Z': int, 'E':float, 'theta': float},
    'DCSb_Rayl': {'Z': int, 'E':float, 'theta': float},
    'DCSb_Compt': {'Z': int, 'E':float, 'theta': float},
    'DCSP_Thoms': {'theta': float, 'phi': float},
    'DCSP_KN': {'E': float, 'theta': float, 'phi': float},
    'DCSP_Rayl': {'Z': int, 'E': float, 'theta': float, 'phi': float},
    'DCSP_Compt': {'Z': int, 'E': float, 'theta': float, 'phi': float},
    'DCSPb_Rayl': {'Z': int, 'E': float, 'theta': float, 'phi': float},
    'DCSPb_Compt': {'Z': int, 'E': float, 'theta': float, 'phi': float},
    'FF_Rayl': {'Z': int, 'q': float},
    'SF_Compt': {'Z': int, 'q': float},
    'MomentTransf': {'E': float, 'theta': float},
    'LineEnergy': {'Z': int, 'line': int},
    'FluorYield': {'Z': int, 'shell': int},
    'CosKronTransProb': {'Z': int, 'trans': int},
    'EdgeEnergy': {'Z': int, 'shell': int},
    'JumpFactor': {'Z': int, 'shell': int},
    'CS_FluorLine': {'Z': int, 'line': int, 'E': float},
    'CSb_FluorLine': {'Z': int, 'line': int, 'E': float},
    'CS_FluorShell': {'Z': int, 'shell': int, 'E': float},
    'CSb_FluorShell': {'Z': int, 'shell': int, 'E': float},
    'RadRate': {'Z': int, 'line': int},
    'ComptonEnergy': {'E0': float, 'theta': float},
    'Fi': {'Z': int, 'E': float},
    'Fii': {'Z': int, 'E': float},
    'CS_Photo_Total': {'Z': int, 'E': float},
    'CSb_Photo_Total': {'Z': int, 'E': float},
    'CS_Photo_Partial': {'Z': int, 'shell': int, 'E': float},
    'CSb_Photo_Partial': {'Z': int, 'shell': int, 'E': float},
    'CS_Total_Kissel': {'Z': int, 'E': float},
    'CSb_Total_Kissel': {'Z': int, 'E': float},
    'ComptonProfile': {'Z': int, 'pz': float},
    'ComptonProfile_Partial': {'Z': int, 'shell': int, 'pz': float},
    'ElectronConfig': {'Z': int, 'shell': int},
    'ElectronConfig_Biggs': {'Z': int, 'shell': int},
    'AtomicLevelWidth': {'Z': int, 'shell': int},
    'AugerRate': {'Z': int, 'auger_trans': int},
    'AugerYield': {'Z': int, 'shell': int},
    'CS_FluorLine_Kissel': {'Z': int, 'line': int, 'E': float},
    'CSb_FluorLine_Kissel': {'Z': int, 'line': int, 'E': float},
    'CS_FluorLine_Kissel_Cascade': {'Z': int, 'line': int, 'E': float},
    'CSb_FluorLine_Kissel_Cascade': {'Z': int, 'line': int, 'E': float},
    'CS_FluorLine_Kissel_Nonradiative_Cascade': {'Z': int, 'line': int, 'E': float},
    'CSb_FluorLine_Kissel_Nonradiative_Cascade': {'Z': int, 'line': int, 'E': float},
    'CS_FluorLine_Kissel_Radiative_Cascade': {'Z': int, 'line': int, 'E': float},
    'CSb_FluorLine_Kissel_Radiative_Cascade': {'Z': int, 'line': int, 'E': float},
    'CS_FluorLine_Kissel_no_Cascade': {'Z': int, 'line': int, 'E': float},
    'CSb_FluorLine_Kissel_no_Cascade': {'Z': int, 'line': int, 'E': float},
    'CS_FluorShell_Kissel': {'Z': int, 'shell': int, 'E': float},
    'CSb_FluorShell_Kissel': {'Z': int, 'shell': int, 'E': float},
    'CS_FluorShell_Kissel_Cascade': {'Z': int, 'shell': int, 'E': float},
    'CSb_FluorShell_Kissel_Cascade': {'Z': int, 'shell': int, 'E': float},
    'CS_FluorShell_Kissel_Nonradiative_Cascade': {'Z': int, 'shell': int, 'E': float},
    'CSb_FluorShell_Kissel_Nonradiative_Cascade': {'Z': int, 'shell': int, 'E': float},
    'CS_FluorShell_Kissel_Radiative_Cascade': {'Z': int, 'shell': int, 'E': float},
    'CSb_FluorShell_Kissel_Radiative_Cascade': {'Z': int, 'shell': int, 'E': float},
    'CS_FluorShell_Kissel_no_Cascade': {'Z': int, 'shell': int, 'E': float},
    'CSb_FluorShell_Kissel_no_Cascade': {'Z': int, 'shell': int, 'E': float},
    'PL1_pure_kissel': {'Z': int, 'E': float},
    'PL1_rad_cascade_kissel': {'Z': int, 'E': float, 'PK': float},
    'PL1_auger_cascade_kissel': {'Z': int, 'E': float, 'PK': float},
    'PL1_full_cascade_kissel': {'Z': int, 'E': float, 'PK': float},
    'PL2_pure_kissel': {'Z': int, 'E': float, 'PL1': float},
    'PL2_rad_cascade_kissel': {'Z': int, 'E': float, 'PK': float, 'PL1': float},
    'PL2_auger_cascade_kissel': {'Z': int, 'E': float, 'PK': float, 'PL1': float},
    'PL2_full_cascade_kissel': {'Z': int, 'E': float, 'PK': float, 'PL1': float},
    'PL3_pure_kissel': {'Z': int, 'E': float, 'PL1': float, 'PL2': float},
    'PL3_rad_cascade_kissel': {'Z': int, 'E': float, 'PK': float, 'PL1': float, 'PL2': float},
    'PL3_auger_cascade_kissel': {'Z': int, 'E': float, 'PK': float, 'PL1': float, 'PL2': float},
    'PL3_full_cascade_kissel': {'Z': int, 'E': float, 'PK': float, 'PL1': float, 'PL2': float},
    'PM1_pure_kissel': {'Z': int, 'E': float},
    'PM1_rad_cascade_kissel': {'Z': int, 'E': float, 'PK': float, 'PL1': float, 'PL2': float, 'PL3': float},
    'PM1_auger_cascade_kissel': {'Z': int, 'E': float, 'PK': float, 'PL1': float, 'PL2': float, 'PL3': float},
    'PM1_full_cascade_kissel': {'Z': int, 'E': float, 'PK': float, 'PL1': float, 'PL2': float, 'PL3': float},
    'PM2_pure_kissel': {'Z': int, 'E': float, 'PM1': float},
    'PM2_rad_cascade_kissel': {'Z': int, 'E': float, 'PK': float, 'PL1': float, 'PL2': float, 'PL3': float, 'PM1': float},
    'PM2_auger_cascade_kissel': {'Z': int, 'E': float, 'PK': float, 'PL1': float, 'PL2': float, 'PL3': float, 'PM1': float},
    'PM2_full_cascade_kissel': {'Z': int, 'E': float, 'PK': float, 'PL1': float, 'PL2': float, 'PL3': float, 'PM1': float},
    'PM3_pure_kissel': {'Z': int, 'E': float, 'PM1': float, 'PM2': float},
    'PM3_rad_cascade_kissel': {'Z': int, 'E': float, 'PK': float, 'PL1': float, 'PL2': float, 'PL3': float, 'PM1': float, 'PM2': float},
    'PM3_auger_cascade_kissel': {'Z': int, 'E': float, 'PK': float, 'PL1': float, 'PL2': float, 'PL3': float, 'PM1': float, 'PM2': float},
    'PM3_full_cascade_kissel': {'Z': int, 'E': float, 'PK': float, 'PL1': float, 'PL2': float, 'PL3': float, 'PM1': float, 'PM2': float},
    'PM4_pure_kissel': {'Z': int, 'E': float, 'PM1': float, 'PM2': float, 'PM3': float},
    'PM4_rad_cascade_kissel': {'Z': int, 'E': float, 'PK': float, 'PL1': float, 'PL2': float, 'PL3': float, 'PM1': float, 'PM2': float, 'PM3': float},
    'PM4_auger_cascade_kissel': {'Z': int, 'E': float, 'PK': float, 'PL1': float, 'PL2': float, 'PL3': float, 'PM1': float, 'PM2': float, 'PM3': float},
    'PM4_full_cascade_kissel': {'Z': int, 'E': float, 'PK': float, 'PL1': float, 'PL2': float, 'PL3': float, 'PM1': float, 'PM2': float, 'PM3': float},
    'PM5_pure_kissel': {'Z': int, 'E': float, 'PM1': float, 'PM2': float, 'PM3': float, 'PM4': float},
    'PM5_rad_cascade_kissel': {'Z': int, 'E': float, 'PK': float, 'PL1': float, 'PL2': float, 'PL3': float, 'PM1': float, 'PM2': float, 'PM3': float, 'PM4': float},
    'PM5_auger_cascade_kissel': {'Z': int, 'E': float, 'PK': float, 'PL1': float, 'PL2': float, 'PL3': float, 'PM1': float, 'PM2': float, 'PM3': float, 'PM4': float},
    'PM5_full_cascade_kissel': {'Z': int, 'E': float, 'PK': float, 'PL1': float, 'PL2': float, 'PL3': float, 'PM1': float, 'PM2': float, 'PM3': float, 'PM4': float},

    'CS_Total_CP': {'compound': str, 'E': float},
    'CS_Photo_CP': {'compound': str, 'E': float},
    'CS_Rayl_CP': {'compound': str, 'E': float},
    'CS_Compt_CP': {'compound': str, 'E': float},
    'CS_Energy_CP': {'compound': str, 'E': float},
    'CSb_Total_CP': {'compound': str, 'E': float},
    'CSb_Photo_CP': {'compound': str, 'E': float},
    'CSb_Rayl_CP': {'compound': str, 'E': float},
    'CSb_Compt_CP': {'compound': str, 'E': float},
    'DCS_Rayl_CP': {'compound': str, 'E': float, 'theta': float},
    'DCS_Compt_CP': {'compound': str, 'E': float, 'theta': float},
    'DCSb_Rayl_CP': {'compound': str, 'E': float, 'theta': float},
    'DCSb_Compt_CP': {'compound': str, 'E': float, 'theta': float},
    'DCSP_Rayl_CP': {'compound': str, 'E': float, 'theta': float, 'phi': float},
    'DCSP_Compt_CP': {'compound': str, 'E': float, 'theta': float, 'phi': float},
    'DCSPb_Rayl_CP': {'compound': str, 'E': float, 'theta': float, 'phi': float},
    'DCSPb_Compt_CP': {'compound': str, 'E': float, 'theta': float, 'phi': float},
    'CS_Photo_Total_CP': {'compound': str, 'E': float},
    'CS_Total_Kissel_CP': {'compound': str, 'E': float},
    'CSb_Photo_Total_CP': {'compound': str, 'E': float},
    'CSb_Total_Kissel_CP': {'compound': str, 'E': float},
    'Refractive_Index_Re': {'compound': str, 'E': float, 'density': float},
    'Refractive_Index_Im': {'compound': str, 'E': float, 'density': float}
    }

def generate_declaration_for_str(arg_name: str) -> str:
    return f'''CHARACTER (KIND=C_CHAR,LEN=*), INTENT(IN) :: {arg_name} 
        CHARACTER (KIND=C_CHAR), DIMENSION(:), ALLOCATABLE, TARGET :: &
        {arg_name}_F'''

def generate_declaration_for_float(arg_name: str) -> str:
    return f"REAL (C_DOUBLE), INTENT(IN) :: {arg_name}"

def generate_declaration_for_int(arg_name: str) -> str:
    return f"INTEGER (C_INT), INTENT(IN) :: {arg_name}"

def generate_iface_declaration_for_str(arg_name: str) -> str:
    return f"TYPE (C_PTR), INTENT(IN), VALUE :: {arg_name}"

def generate_iface_declaration_for_float(arg_name: str) -> str:
    return f"REAL (C_DOUBLE), INTENT(IN), VALUE :: {arg_name}"

def generate_iface_declaration_for_int(arg_name: str) -> str:
    return f"INTEGER (C_INT), INTENT(IN), VALUE :: {arg_name}"




GENERATE_DECLARATION = {
    str: generate_declaration_for_str,
    float: generate_declaration_for_float,
    int: generate_declaration_for_int
}

GENERATE_IFACE_DECLARATION = {
    str: generate_iface_declaration_for_str,
    float: generate_iface_declaration_for_float,
    int: generate_iface_declaration_for_int
}

def generate_preprocess_for_str(arg_name: str) -> str:
    return f'''CALL stringF2C({arg_name}, {arg_name}_F)
'''

GENERATE_PREPROCESS = {
    str: generate_preprocess_for_str,
    float: lambda arg_name: "",
    int: lambda arg_name: ""
}

GENERATE_CALL = {
    str: lambda arg_name: f"C_LOC({arg_name}_F)",
    float: lambda arg_name: f"{arg_name}",
    int: lambda arg_name: f"{arg_name}"
}

def process_function(name: str) -> str:

    args = XRL_FUNCTIONS[name]

    arg_list = ', '.join(args.keys())

    arg_declarations = '\n        '.join([GENERATE_DECLARATION[arg_type](arg_name) for arg_name, arg_type in args.items()])

    arg_preprocess = ''.join([GENERATE_PREPROCESS[arg_type](arg_name) for arg_name, arg_type in args.items()])

    arg_calls =  ', '.join([GENERATE_CALL[arg_type](arg_name) for arg_name, arg_type in args.items()])
    arg_iface_declarations = '\n                '.join([GENERATE_IFACE_DECLARATION[arg_type](arg_name) for arg_name, arg_type in args.items()])

    return f'''
FUNCTION {name}({arg_list}, error) RESULT(rv)
        USE, INTRINSIC :: ISO_C_BINDING
        USE, INTRINSIC :: ISO_FORTRAN_ENV
        IMPLICIT NONE

        {arg_declarations}
        REAL (C_DOUBLE) :: rv

        TYPE(xrl_error), POINTER, OPTIONAL :: error
        TYPE (C_PTR) :: errorPtr, errorPtrLoc
        TARGET :: errorPtr

        INTERFACE
            FUNCTION {name}C({arg_list}, error) &
            BIND(C,NAME='{name}')
                USE, INTRINSIC :: ISO_C_BINDING
                IMPLICIT NONE
                {arg_iface_declarations}
                REAL (C_DOUBLE) :: {name}C
                TYPE (C_PTR),INTENT(IN),VALUE :: error
            ENDFUNCTION {name}C
        ENDINTERFACE

        errorPtr = C_NULL_PTR
        errorPtrLoc = C_NULL_PTR

        IF (PRESENT(error)) THEN
                IF (.NOT. ASSOCIATED(error)) THEN
                        errorPtrLoc = C_LOC(errorPtr)
                ELSE
                        ! print warning
                        WRITE (error_unit, '(A)') & 
                        'error POINTER must be disassociated!'
                ENDIF
        ENDIF
        {arg_preprocess}
        rv = {name}C({arg_calls}, errorPtrLoc)

        IF (C_ASSOCIATED(errorPtr)) THEN
                CALL process_error(errorPtr, error)
        ENDIF
ENDFUNCTION {name}
'''


with open('xraylib_wrap_generated.F90', 'w') as f:
    f.write('! This file has been generated automatically using generate-code.py\n\n')
    f.write('\n'.join([process_function(function) for function in XRL_FUNCTIONS.keys()]))



import os
if "EXTRA_DLL_SEARCH_PATHS" in os.environ and hasattr(os, "add_dll_directory"):
    for path in os.environ["EXTRA_DLL_SEARCH_PATHS"].split(os.pathsep):
        os.add_dll_directory(path)
import unittest
import xraylib


class TestAtomicWeight(unittest.TestCase):
    def test_Fe(self):
        weight = xraylib.AtomicWeight(26)
        self.assertAlmostEqual(weight, 55.850)

    def test_U(self):
        weight = xraylib.AtomicWeight(92)
        self.assertAlmostEqual(weight, 238.070)

    def test_bad_Z(self):
        with self.assertRaises(ValueError):
            width = xraylib.AtomicWeight(185)

if __name__ == '__main__':
    unittest.main(verbosity=2)




#!/usr/bin/env python3


import urllib.request
from io import StringIO
import lxml.etree as ET
import re
import xraylib as xrl

xrl.SetErrorMessages(0)

lbl_url = "http://nucleardata.nuclear.lu.se/toi/nuclide.asp?iZA="

nuclide_codes = sorted(("260055", "940238", "960244", "480109", "530125", "950241", "640153", "270057", "560133", "550137"))

with open("../../src/xraylib-radionuclides-internal.h", "w") as output_int, open("../../include/xraylib-radionuclides.h", "w") as output_header:


    header_begin = '''/*
Copyright (c) 2014-2018, Tom Schoonjans
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
    * The names of the contributors may not be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY Tom Schoonjans ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Tom Schoonjans BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

/*
 * This file was automatically generated by nuclides.pl
 * Modify at your own risk...
 */


#ifndef XRAYLIB_RADIO_NUCLIDES_H
#define XRAYLIB_RADIO_NUCLIDES_H

#include "xraylib-error.h"

/* 
 * name: a string containing the mass number (A), followed by the chemical element (e.g. 55Fe)
 * Z: atomic number of the radionuclide
 * A: mass number of the radionuclide
 * N: number of neutrons of the radionuclide
 * Z_xray: atomic number of the nuclide after decay, and which should be used in calculating the energy of the emitted X-ray lines
 * nXrays: number of emitted characteristic X-rays
 * XrayLines: array of *_LINE macros, identifying the emitted X-rays
 * XrayIntensities: array of photons per disintegration, one value per emitted X-ray
 * nGammas: number of emitted gamma-rays
 * GammaEnergies: array of emitted gamma-ray energies
 * GammaIntensities: array of emitted gamma-ray photons per disintegration
 */

struct radioNuclideData{
	char *name;
	int Z;
	int A;
	int N;
	int Z_xray;
	int nXrays;
	int *XrayLines;
	double *XrayIntensities;
	int nGammas;
	double *GammaEnergies;
	double *GammaIntensities;
};


/*
 *
 * Returns a pointer to a newly allocated struct containing
 * the requested radionuclide on success, or NULL when the radionuclide 
 * was not found in the list. The radionuclide is requested by providing
 * its name as argument to the function. For a list of available names,
 * use GetRadioNuclideList.
 *
 * The returned struct should be freed after usage with FreeRadioNuclideData.
 *
 */
XRL_EXTERN
struct radioNuclideData *GetRadioNuclideDataByName(const char radioNuclideString[], xrl_error **error);

/*
 *
 * Returns a pointer to a newly allocated struct containing
 * the requested radionuclide on success, or NULL when the radionuclide
 * was not found in the list. The radionuclide is requested by providing
 * its index in the internal table to the function. Typically this would
 * be done using the RADIO_NUCLIDE_* macros in this file.
 *
 * The returned struct should be freed after usage with FreeRadioNuclideData.
 *
 */
XRL_EXTERN
struct radioNuclideData *GetRadioNuclideDataByIndex(int radioNuclideIndex, xrl_error **error);

/*
 *
 * Returns a NULL-terminated array of strings of all the radionuclides in the
 * internal table. If nRadioNuclides is not NULL, it shall receive the number 
 * of radionuclides.
 *
 * The returned array should be freed firstly by using xrlFree to deallocate
 * all individual strings, and subsequently by using xrlFree to deallocate the array
 *
 */
XRL_EXTERN
char **GetRadioNuclideDataList(int *nRadioNuclides, xrl_error **error);

/*
 *
 * Deallocates a pointer to a radioNuclideData struct completely.
 * It is recommended to set the pointer to NULL after calling this function.
 *
 */
XRL_EXTERN
void FreeRadioNuclideData(struct radioNuclideData *radioNuclideData);

'''

    output_header.write(header_begin)

    header_begin2 = '''/*
Copyright (c) 2014-2018, Tom Schoonjans
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
    * The names of the contributors may not be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY Tom Schoonjans ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Tom Schoonjans BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

/*
 * This file was automatically generated by nuclides.pl
 * Modify at your own risk...
 */

#include "xraylib.h"
#include "xraylib-error-private.h"

'''
    output_int.write(header_begin2)


    nuclideData = []

    for nuclideIndex, nuclide_code in enumerate(nuclide_codes):
        print(nuclide_code)
        url = lbl_url + nuclide_code
        print("url: {}".format(url))

        try:
            response = urllib.request.urlopen(url)
            html = response.read().decode("utf-8")

            # fix html file
            html = re.sub(r"(<\/FONT>\r\n)<\/FONT>", r"\1", html)
            html = re.sub(r"(Tables:<\/A>)<\/FONT>", r"\1", html)
            html = re.sub(r"(ENSDF data:<\/A>)<\/FONT>", r"\1", html)
            html = re.sub(r"(Java applets:<\/A>)<\/FONT>", r"\1", html)
            html = re.sub(r"(TORI Data \(1999\))(<\/font>)(<\/I>)", r"\1\3\2", html)
            html = re.sub(r"(<\/B>)(<\/FONT>)(<P>)", r"\2\1\3", html)
            html = re.sub(r"&nbsp;<I>.+<\/I>&nbsp;", "", html)
            html = re.sub(r"&nbsp;<i>.+<\/i>&nbsp;", "", html)
            html = re.sub(r"/&nbsp;", "", html)
            html = re.sub(r"(<TD>)<\d.+(<\/TD>)", r"\1\2", html)

            parser = ET.HTMLParser()
            tree = ET.parse(StringIO(html), parser)

            xpath_Z = "/html/body/center/table/tr[3]/td[2]/center/table[1]/caption/table/tr[2]/th[1]/font/text()";
            xpath_A = "/html/body/center/table/tr[3]/td[2]/center/table[1]/caption/table/tr[1]/th[1]/font/text()";
            xpath_N = "/html/body/center/table/tr[3]/td[2]/center/table[1]/caption/table/tr[2]/th[2]/font/text()";

            nuclideDataSingle = dict()

            Z = tree.xpath(xpath_Z)[0]
            A = tree.xpath(xpath_A)[0]
            N = tree.xpath(xpath_N)[0]
            print("Z: {}".format(Z))
            print("A: {}".format(A))
            print("N: {}".format(N))

            nuclideDataSingle['Z'] = Z
            nuclideDataSingle['A'] = A
            nuclideDataSingle['N'] = N
            nuclideDataSingle['name'] = "{}{}".format(A, xrl.AtomicNumberToSymbol(int(Z)))

            ##gamma stuff
            xpath_gamma = "/html/body/center/table/tr[3]/td[2]/center/table[3]/tr[position() >= 4 and position() < count(/html/body/center/table/tr[3]/td[2]/center/table[3]/tr)]"
            gamma_nodes = tree.xpath(xpath_gamma)

            print("gamma nodes number: {}".format(len(gamma_nodes)))

            gamma_energies = []
            gamma_intensities = []

            for node in gamma_nodes:
                try:
                    gamma_intensity = node.xpath("td[2]/text()")[0]
                    gamma_intensity = float(gamma_intensity)
                    gamma_energies.append(node.xpath("td[1]/text()")[0].strip())
                    gamma_intensities.append("{:g}".format(gamma_intensity / 100.0))
                except Exception:
                    pass

            nuclideDataSingle['nGammas'] = len(gamma_energies)
            nuclideDataSingle['gammaEnergies'] = gamma_energies
            nuclideDataSingle['gammaIntensities'] = gamma_intensities
            print("nGammas: {}".format(nuclideDataSingle['nGammas']))


            ##X-rays
            xpath_xray= "/html/body/center/table/tr[3]/td[2]/center/table[4]/tr[position() >= 4 and position() <= count(/html/body/center/table/tr[3]/td[2]/center/table[4]/tr)]"
            xray_nodes = tree.xpath(xpath_xray)

            print("xray nodes number: {}".format(len(xray_nodes)))

            xray_lines = []
            xray_intensities = []

            # extract X-ray element
            Z_xray = xray_nodes[0].xpath("td[3]/text()")[0][0:2]
            if not Z_xray[1].isalpha() or not Z_xray[1].islower():
                Z_xray = Z_xray[0]
            print("Z_xray: {}".format(Z_xray))
            nuclideDataSingle['Z_xray'] = xrl.SymbolToAtomicNumber(Z_xray)

            for node in xray_nodes:
                xray_intensity = node.xpath("td[2]/text()")[0]
                line = node.xpath("td[3]/text()")[0]
                line = line.replace(Z_xray, "")
                line_major = line.upper()
                line_minor = node.xpath("td[3]/sub/*/text()")[0].upper()
                try:
                    line_micro = node.xpath("td[3]/sub/text()")[0].upper()
                except IndexError:
                    line_micro = ""
                line = line_major + line_minor + line_micro + "_LINE"
                line = line.strip()
                print("line: {}".format(line))
                try:
                    xray_intensity = float(xray_intensity)
                    xray_energy = xrl.LineEnergy(nuclideDataSingle['Z_xray'], getattr(xrl, line))
                    if xray_energy == 0.0:
                        raise Exception("LineEnergy not found for {} {}", nuclideDataSingle['Z_xray'], line)
                    print("Energies: {}  {}".format(node.xpath("td[1]/text()")[0], xray_energy))
                    xray_intensities.append("{:g}".format(xray_intensity/100.0))
                    xray_lines.append(line)
                except Exception as e:
                    print("Exception: " + str(e))

            nuclideDataSingle['nXrays'] = len(xray_intensities)
            print("nXrays: {}".format(nuclideDataSingle['nXrays']))
            nuclideDataSingle['xrayIntensities'] = xray_intensities
            nuclideDataSingle['xrayLines'] = xray_lines

            output_header.write("#define RADIO_NUCLIDE_{} {}\n".format(nuclideDataSingle['name'].upper(), nuclideIndex))
            nuclideData.append(nuclideDataSingle)
        except Exception as e:
            print(e)
            raise


    output_int.write("static const int nNuclideDataList = {};\n".format(len(nuclideData)))

    for i, nd in enumerate(nuclideData):
        output_int.write("static int __NuclideDataList_XrayLines_{}[] = {{{}}};\n".format(i, ", ".join(nd['xrayLines'])))
        output_int.write("static double __NuclideDataList_XrayIntensities_{}[] = {{{}}};\n".format(i, ", ".join(nd['xrayIntensities'])))
        output_int.write("static double __NuclideDataList_GammaEnergies_{}[] = {{{}}};\n".format(i, ", ".join(nd['gammaEnergies'])))
        output_int.write("static double __NuclideDataList_GammaIntensities_{}[] = {{{}}};\n".format(i, ", ".join(nd['gammaIntensities'])))

    output_int.write("static const struct radioNuclideData nuclideDataList[] = {\n")
    ls = ["{{\"{}\" ,{}, {}, {}, {}, {}, __NuclideDataList_XrayLines_{}, __NuclideDataList_XrayIntensities_{}, {}, __NuclideDataList_GammaEnergies_{}, __NuclideDataList_GammaIntensities_{}}}".format( \
        nd['name'], \
	nd['Z'], \
        nd['A'], \
        nd['N'], \
        nd['Z_xray'], \
        nd['nXrays'], \
        i, \
        i, \
        nd['nGammas'], \
        i, \
        i) for i, nd in enumerate(nuclideData)]
    output_int.write(",\n".join(ls) + "\n")

    output_int.write("};\n")

    output_header.write("\n#endif\n")





import os
if "EXTRA_DLL_SEARCH_PATHS" in os.environ and hasattr(os, "add_dll_directory"):
    for path in os.environ["EXTRA_DLL_SEARCH_PATHS"].split(os.pathsep):
        os.add_dll_directory(path)
import unittest
import xraylib
import numpy as np



class TestNumpy(unittest.TestCase):
    def _test_np(self, dtype):
        for Z in np.arange(10, 20, dtype=dtype):
            print( xraylib.LineEnergy(Z, xraylib.KL2_LINE))

    def test_np_i16(self):
        self._test_np(np.int16)

    def test_np_i32(self):
        self._test_np(np.int32)

    def test_np_i64(self):
        self._test_np(np.int64)

if __name__ == '__main__':
    unittest.main(verbosity=2)



import os
if "EXTRA_DLL_SEARCH_PATHS" in os.environ and hasattr(os, "add_dll_directory"):
    for path in os.environ["EXTRA_DLL_SEARCH_PATHS"].split(os.pathsep):
        os.add_dll_directory(path)
import unittest
import xraylib


class TestAtomicLevelWidth(unittest.TestCase):
    def test_Fe_K(self):
        width = xraylib.AtomicLevelWidth(26, xraylib.K_SHELL)
        self.assertAlmostEqual(width, 1.19E-3)

    def test_U_N7(self):
        width = xraylib.AtomicLevelWidth(92, xraylib.N7_SHELL)
        self.assertAlmostEqual(width, 0.31E-3)

    def test_bad_Z(self):
        with self.assertRaises(ValueError):
            width = xraylib.AtomicLevelWidth(185, xraylib.K_SHELL)

    def test_bad_shell(self):
        with self.assertRaises(ValueError):
            width = xraylib.AtomicLevelWidth(26, -5)

    def test_invalid_shell(self):
        with self.assertRaises(ValueError):
            width = xraylib.AtomicLevelWidth(26, xraylib.N3_SHELL)

if __name__ == '__main__':
    unittest.main(verbosity=2)
